%******************************************************************************;
%* SAS macro DummyGen vs. 0.0.5a,  5 sep 2012, by Jim Groeneveld, Netherlands *;
%******************************************************************************;
%*                                                                            *;
%*;%MACRO   DummyGen   /* Creation of binary, dummy (indicator) variables     */
/* -------------------------------------------------------------------------- */
/* ---------- A l w a y s   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  Data    =_LAST_     /* Input data set specification                      R */
,  Out     =           /* Output data set specification (may be InputData)  R */
,  OverWrit=N          /* specify Y to overwrite an existing Out dataset    O */
,  Variable=           /* categpry variable in input dataset to be broken   R */
                       /*   up into several binary dummy (censor) variables   */
/*                                                                            */
/* ------ F r e q u e n t l y   s p e c i f i e d   a r g u m e n t s ------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  DummyTot=           /* macro variable name to contain number of dummies  O */
,  DummyLst=           /* macro variable name to contain dummy name list    O */
,  KeyValue=_KeyVar_   /* name of character variable with recoded and SAS     */
                       /*   name compliant names as values for matching goals */
/*                                                                            */
/* ---- O c c a s i o n a l l y   s p e c i f i e d   a r g u m e n t s ----- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  Response= 1         /* Response value for dummy variables, default 1     O */
,  NonResp = 0         /* Non-Response value for dummy variables, default 0 O */
,  Missing = /* 0 */   /* Optional Missing dummy value, default none or 0   O */
/*                                                                            */
/* ---------- R a r e l y   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  Version = 0.0.5a    /* forced version specification for version control  O */
,  CalledBy=DummyGen   /* Macro name of macro or program for err0r report   O */
/*                                                                            */
/* ----------- N e v e r   s p e c i f i e d   a r g u m e n t s ------------ */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
                       /* - - - - - - - - - - - - - - - - - - - - - - - - - - */
,  Debug   = 0         /* Debug level; for testing, debugging purposes only O */
,  ForMyUse=           /* argument for the author's use only, do not use it ! */
)/ DES     =             'Generate dummy (indicator) variables'   /* STORE */  ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : DummyGen.SAS / DummyGen.005                                   *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Main characteristics:                                                      *;
%* ---------------------                                                      *;
%* SAS macro DummyGen generates binary, dummy, indicator variables for all    *;
%* occurring categories of a categorical variable. These can be used as       *;
%* censor variables with logistic regression and various general linear       *;
%* models. They can also easily be used in simple nonparametric designs.      *;
%*                                                                            *;
%* All uniquely named dummy generated variable names (by default) are based   *;
%* on the categorical values of the concerning variable(s). In case of        *;
%* apparent duplicate names random names are generated that can change each   *;
%* time the macro is run on same data.                                        *;
%*                                                                            *;
%* Features and extensions:                                                   *;
%* ------------------------                                                   *;
%* - category values may be converted to SAS name compliant variable names    *;
%*   (allowable characters, digits and underscore, maximum length)            *;
%* - if generated names already exist alternative, random names are generated *;
%* - category variable may be either numeric or character and may include     *;
%*   (discriminated) missing values (dot and space become an underscore)      *;
%* - pass the dummy variable name list to the calling program via a macro     *;
%*   call parameter value, specifying the macro variable name of the list.    *;
%*   From this list the elements, which are the dummy variable names, can be  *;
%*   obtained using function %SCAN (or %SYSFUNC(SCANQ with SAS literal names).*;
%*   The dummy variables should preferably be referred via the macro variable *;
%*   names that store the SAS names, as the SAS names could be generated      *;
%*   randomly and may change every time this macro DummyGen is run. These     *;
%*   names are primarily intended for temporary use.                          *;
%* - the relationship between the original category values and the            *;
%*   corresponding, converted dummy names is stored as a user specified       *;
%*   variable for the KeyValue argument that contains the converted names     *;
%*   for the corresponding category values. This may be used for matching.    *;
%*   The relationship is also given in the variable label of the converted    *;
%*   variable names, which represent the original category values (up to the  *;
%*   maximum allowable length for variable labels).                           *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Programmer : Jim Groeneveld                                                *;
%* Date       :  5 sep 2012                                                   *;
%* Version    : 0.0.5a                                                        *;
%*                                                                            *;
%* SAS version: 9.1.3 and 9.2 and WXP and W7                                  *;
%*                                                                            *;
%* Validation : - (not yet validated, check results, use at your own risk)    *;
%* Disclaimer : The author is not liable in any way for any negative, adverse *;
%*              consequences of the use or misuse of this program / macro.    *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Notes:                                                                     *;
%* ======                                                                     *;
%*                                                                            *;
%* Output:                                                                    *;
%* =======                                                                    *;
%*                                                                            *;
%* Examples:                                                                  *;
%* =========                                                                  *;
%*                                                                            *;
%* Remarks:                                                                   *;
%* ========                                                                   *;
%*                                                                            *;
%* Used auxiliary macros:                                                     *;
%* ----------------------                                                     *;
%* - internal: - [none]                                                       *;
%* - attached: _Random_, _Length_                                             *;
%* - external: %ERR0R                                                         *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of DummyGen:                                                       *;
%* ====================                                                       *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0.0    30 Aug 2012  first working release (generating anything at all)   *;
%* 0.0.1    30 Aug 2012  added DummyTot and DummyLst arguments                *;
%*                       added intra comparisons of regenerated dummy names   *;
%*                         (including &Variable) and regeneration if names    *;
/*                         appear to be identical                             *;
%* 0.0.2    31 Aug 2012  added numerical variable support,                    *;
%*                       improved unique variable name checks (&KeyValue and  *;
%*                         &IndexVal),                                        *; 
%*                       several minor (code) improvements,                   *;
%*                       added standard headers, standard comment, specific   *;
%*                         comment and standard checking code /* arguments */ *;
%*                         Overwrit,_LAST_,Version,CalledBy,Debug,ForMyUse    *;
%* 0.0.3    31 Aug 2012  added KeyValue argument to specify name of dummy     *;
%*                         variable in (it will temporarily exist anyway)     *;
%* 0.0.4    31 Aug 2012  added labeling of dummy variables with values of     *;
%*                         original &Variable                                 *;
%* 0.0.5     4 Sep 2012  added all checks for duplicate variable names and    *;
%*                         reassign random names and recheck duplicates       *;
%* 0.0.5a    5 Sep 2012  added macro %ERR0R in external macro list in header  *;
%*                                                                            *;
%* Future features: (improvements, extensions and bug fixes)                  *;
%* ================                                                           *;
%* - modify way of checking for duplicate names (not so much in data step)    *;
%* - optionally fixed prefix and sequence number                              *;
%*   just array with index value --> Dummy1, Dummy2, etc.                     *;
%* - optionally SAS name literals with VALIDVARNAME=ANY (macro variables:     *;
%*                                                    "&MacroVal"n)           *;
%* - instead of random time based variable names use deduced SAS name         *;
%*   compliant variable names with sequence numbers like with macro MR2RM     *;
%* . variable list, instead of single variable (what with DummyTot, DummyLst?)*;
%* . maximum number of categories (with varlist) ???                          *;
%*                                                                            *;
%******************************************************************************;

  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN %GOTO Finish;

  %IF (&Debug EQ 0) %THEN %DO; %* default;
    OPTIONS NOMPRINT NOMERROR NOSERROR NOMLOGIC NOSYMBOLGEN NOMACROGEN NOSPOOL;
  %END;
  %ELSE %IF (&Debug EQ 1 OR &Debug GT 11) %THEN %DO;
    OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN SPOOL;
  %END;
  %* else if &Debug is empty then no action;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* LOCAL macro variables declarations ;
%*____________________________________;

  %LOCAL I J IndexVal /* array element indexes */
  MacName  /* This macros external name: MR2RM */
  MacroVs  /* Version number major.minor.patch of macro: 0.0.5a*/
  MacDate  /* Date of this version of macro:  5 sep 2012 */
  ErrCount /* number of erroneously specified arguments */
  DataLib  /* Library name of DataFile (for splitting Lib and Set) */
  DataSet  /* Dataset name of DataFile (for splitting Lib and Set) */
  VarNames /* macro variable list of variable names in dataset &Data */
  VarCount /* number of variables in dataset &Data */
  DupList2 /* list of duplicate variables in list 2 */
  Recoded DumNames /* temporary dataset names, full / unique keys */
  Allowed  /* allowed characters in SAS names (not SAS name literals) 8/
  Letters  /* 26 (lowercase) letters */
  Digits   /* 10 digits */
  DumTotal /* number of levels of category variable */
  _DumLst_ /* alternative name for &DummyLst if not specified */
  ValueKey /* argument KeyValue specified (or not), 1 or 0 */
  VarType  /* Type of &Variable: C or N */
  ;

%* Stand-in variable and dataset names;
  %LET MacName  = DummyGen;
  %LET MacroVs  = 0.0.5a  ;
  %LET MacDate  = %STR( 5 sep 2012);

  %LET Digits   = 0123456789;
  %LET Letters  = abcdefghijklmnopqrstuvwxyz;
  %LET Allowed  = %UPCASE(&Letters)_&Letters.&Digits;

  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT |  Macro &MacName, vs &MacroVs, by Jim Groeneveld, &MacDate  |;
  %PUT |      Generation of binary, dummy, indicator variables      |;
  %PUT |____________________________________________________________|;
  %PUT;

  %LET ErrCount = 0;

  %IF (NOT %LENGTH(%SUPERQ(Data))) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Data argument not specified, macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (NOT %SYSFUNC(EXIST(&Data))) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Dataset Data=&Data does not exist,;
    %PUT                      macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;

  %IF ( %UPCASE(&Overwrit) NE YES AND %UPCASE(&Overwrit) NE NO
    AND %UPCASE(&Overwrit) NE Y AND %UPCASE(&Overwrit) NE N) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Overwrit argument not specified or illegal: &Overwrit,;
    %PUT .                      macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %IF (NOT %LENGTH(%SUPERQ(Out))) %THEN %LET Out = &Data;
  %ELSE %IF ( %SYSFUNC(EXIST(&Out)) AND %FirstCh(&OverWrit) NE Y )
    %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Dataset Out=&Out exists and OverWrit is not Y,;
    %PUT                      macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
%* @ @ @ @ @ @ @ @ @  further error checks on intput here;

  %IF (NOT %LENGTH(%SUPERQ(Response))) %THEN %LET Response = .;
  %IF (NOT %LENGTH(%SUPERQ(NonResp ))) %THEN %LET NonResp  = .;
/*%IF (NOT %LENGTH(%SUPERQ(DummyTot))) %THEN %LET DummyTot = _DumTot_;*/
  %IF (NOT %LENGTH(%SUPERQ(DummyLst))) %THEN %LET DummyLst = _DumLst_;

%* @ @ @ @ @ @ @ @ @  feedback here;

  %IF (%LENGTH(%SUPERQ(Version)) AND &Version NE &MacroVs) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Specified version &Version does not match &MacroVs,;
    %PUT                        macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
    %GOTO Finish;
  %END;

%* Process keyword _LAST_ with both Data and Out;
  %IF (%UPCASE(&Data) EQ _LAST_) %THEN %LET Data = &SYSLAST;
  %IF (%UPCASE(&Out) EQ _LAST_) %THEN %LET Out = &SYSLAST;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

%* Process multiple dataset specifications from keyword &Data ;
  %IF (%LENGTH ( %SCAN ( &Data, 2, . ))) %THEN %DO;
    %LET DataLib = %SCAN ( &Data, 1, . );
    %LET Dataset = %SCAN ( &Data, 2, . );
  %END;
  %ELSE %DO;
    %LET DataLib = WORK; /*%SYSFUNC(GETOPTION(WORK));*/ %* (WORK);
    %LET Dataset = %SCAN ( &Data, 1, . );
  %END;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

  %_Length_; %* Determine and set version dependent SAS name lengths;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* Assign dataset names and check whether existing, reassign and check ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
  %LET Recoded  = %_Random_ (NameType=Dataset); %* different random result every time;
  %DO %UNTIL (%UPCASE(&Recoded) NE %UPCASE(&DumNames));
    %LET DumNames = %_Random_ (NameType=Dataset); %* different random result every time;
  %END;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* K E E P   U N I Q U E   V A L U E S   O F   V A R I A B L E ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* to create a macro list of variable names and its amount.    ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
  PROC SORT DATA=&Data (KEEP=&Variable) OUT=&DumNames NODUPKEY; BY &Variable; RUN;

%* Make macro variable list of variables names in original dataset;
  PROC SQL NOPRINT;
    SELECT NAME INTO :VarNames SEPARATED BY ' '  /* variable name list */
      FROM SASHELP.VCOLUMN
      WHERE UPCASE(LIBNAME) EQ UPCASE("&DataLib")
        AND UPCASE(MemName) EQ UPCASE("&Dataset");
    SELECT COUNT(*) INTO :VarCount               /* number of elements */
      FROM SASHELP.VCOLUMN
      WHERE UPCASE(LIBNAME) EQ UPCASE("&DataLib")
        AND UPCASE(MemName) EQ UPCASE("&Dataset");
/* determine number of records in &DumNames, the number of categories */
    SELECT COUNT(*) INTO: DumTotal FROM &DumNames;      /* number of records, dummies  */
  QUIT;

%*put ===>> VarNames=&VarNames; * develop and debug;

%* - - - - - - - - ;
%* Variable checks ;
%* - - - - - - - - ;

%* Check KeyValue specification;
  %IF (%LENGTH(%SUPERQ(KeyValue))) %THEN %DO;
    %LET ValueKey = 1;
%* compare specified KeyValue with input dataset variable names ;
    %CrossChk (LstName1=Input dataset variables, LstName2=KeyValue,
              VarList1 = &VarNames, VarList2 = &KeyValue, Result=ErrCount);
  %END;
  %ELSE %DO; %* No KeyValue specified, create &KeyValue and assure uniqueness;
    %LET ValueKey = 0;
    %DO %UNTIL (NOT %LENGTH(%SUPERQ(DupList2))); %* can only be duplicate &KeyValue;
    %LET KeyValue  = %_Random_;
      %CrossChk (LstName1=Input dataset variables, LstName2=KeyValue, DupNmbrs=DupList2,
                VarList1 = &VarNames, VarList2 = &KeyValue, Result=ErrCount);
    %END;
  %END;

  %IF ( &ErrCount GT 0 ) %THEN %GOTO Aborting;

  %DO %UNTIL (NOT %LENGTH(%SUPERQ(DupList2))); %* can only be duplicate &IndexVal;
    %LET IndexVal = %_Random_; %* create &IndexVal and assure uniqueness;
%put --->> IndexVal=&IndexVal;
    %CrossChk (LstName1=Input dataset variables, LstName2=KeyValue, DupNmbrs=DupList2,
              VarList1 = &VarNames &KeyValue, VarList2 = &IndexVal );
  %END;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* C O N V E R T   U N I Q U E   V A L U E S   T O   S A S   N A M E S     ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* If &Variable is numeric then recode it to a new character variable with ;
%* a random, unique SAS compliant name. If it is character then convert it ;
%* to a random legal SAS name. Assign those to the variable &KeyValue and  ;
%* subsequently to dummy macro variables standing for new dummy variables. ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* Doing so check all converted values against each other and against all  ;
%* other variable names in the original dataset &Data. If duplicate names  ;
%* occur then assign another random or numbered name to the variable to be.;
%* Finally merge the generated dummy names (in &KeyValue) with the original;
%* data for further processing into dummy variables with response values.  ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

  PROC FORMAT; VALUE $BEST; RUN; * to avoid NOTE on not found or not loaded;
  DATA &DumNames /*(DROP=_NewVar_)*/;
    SET &DumNames /*NOBS=_DumTot_*/;
    LENGTH &KeyValue $256;
    LABEL &KeyValue = 'dummy names used for matching purposes';
    IF (_N_ EQ 1) THEN CALL SYMPUT ('VarType', VTYPE(&Variable)); * just once;
%* Numeric &Variable;
    IF (VTYPE(&Variable) EQ 'N') THEN DO;
      &KeyValue = '_' || TRANSLATE(LEFT(PUT(&Variable,BEST30.14)),'_','-E.');
    END;

%* Character &Variable;
    IF (VTYPE(&Variable) EQ 'C') THEN DO;
%* Convert &KeyValue to contain only SAS name allowed characters ;
%* Remove illegal characters, keep legal ones, truncate at allowed number of characters;
      IF (INDEX("%UPCASE(&Letters)_&Letters", SUBSTRN(LEFT(&Variable),1,1))) THEN
/* &Variable name starts with a letter or underscore and is not empty */
        &KeyValue = SUBSTRN(COMPRESS (LEFT(&Variable), "&Allowed", 'K'), 1, &MaxName);
      ELSE /* &Variable name starts with a non-allowed character or is not empty */
        &KeyValue = '_' || SUBSTRN(COMPRESS (LEFT(&Variable), "&Allowed", 'K'), 1, &MaxName-1);
    END;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* M U T U A L   C O M P A R I S O N   O F   A L L   G E N E R A T E D ;
%* D U M M Y   V A R I A B L E   N A M E S   ( + & V a r i a b l e + ) ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* Verify uniquely generated variable names, if not, make them unique. ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
%* Check for duplicate name values with dataset variable names (&VarNames), ;
%* and two others (&KeyValue, &IndexVal) and with already converted values. ;
%* If duplicate found then redefine and recheck in the DO UNTIL loop.       ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
    _OldVar_ = "&VarNames &KeyValue &IndexVal"; * {&VarCount+2} ; * length set;
    ARRAY _NewVar_ [&DumTotal] $&MaxName; * to store dummy variable names ;
    RETAIN _NewVar_;
    DO UNTIL (Redefine EQ 0);
      Redefine = 0;
* Existing variables from original dataset;
      DO _I_ = 1 TO &VarCount+2;
        IF (UPCASE(&KeyValue) EQ UPCASE(SCANQ(_OldVar_,_I_,' '))) THEN Redefine = 1;
      END;
* New variables that are the dummies;
      _NewVar_[_N_] = &KeyValue;
      DO _I_ = 1 TO _N_-1;
        IF (UPCASE(&KeyValue) EQ UPCASE(_NewVar_[_I_])) THEN Redefine = 1;
      END;
* If duplicate names found then reassign random name and recheck;
      IF (Redefine) THEN DO;
        CALL SLEEP (1,.01); * wait for some time before generating time based random name;
        &KeyValue = '_' || LEFT(SUBSTRN(TRANSLATE(TIME(),'_','.:'),1,&MaxName-1)); * random time;
      END;
    END; * do until;
  RUN;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* W A R N I N G :   randomly generated unique variable names change     ;
%* every time the macro is run. Do not refer to them directly, but only  ;
%* indirectly via the macro variable name list using the function %SCAN  ;
%* or the concerning value of the variable &KeyValue in dataset &Recoded.;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* G E N E R A T E   D U M M Y   V A R I A B L E   N A M E S   F R O M   V A L U E S ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* Generate legal SAS variable names, prefixed by '_', into macro variables Dummy#,  ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
  PROC SQL;                                             /*-----------------------------*/
  /*SELECT COUNT(*) INTO: DumTotal FROM &DumNames;*/    /* number of records, dummies  */
    SELECT &KeyValue                                    /* SAS name compliant var name */
      INTO :Dummy1 - :Dummy%EVAL(&DumTotal)             /* store in macro variables    */
      FROM &DumNames;                                   /* EVAL removes leading spaces */
  QUIT;                                                 /*-----------------------------*/
%* - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* A &Variable`s value may be missing (character ' ');
%* the dummy name in such an instance will be just _ ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - ;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* R E T U R N   D U M M Y   V A R I A B L E   N U M B E R   &   L I S T ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* Return number of dummy variables and dummy variable list to the       ;
%* calling program via macro arguments, specifying macro variable that   ;
%* must exist already, at least empty, e.g. DummyLst = ListName to store ;
%* generated dummy variable list in outer macro variable ListName. This  ;
%* can further be processed using %SCAN extracting individual names.     ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
  %IF (%LENGTH(%SUPERQ(DummyTot))) %THEN %LET &DummyTot = &DumTotal; * calling level;
  %IF (%LENGTH(%SUPERQ(DummyLst))) %THEN %DO;
    %LET &DummyLst = ;
    %DO I = 1 %TO &DumTotal;
      %LET &DummyLst = &&&DummyLst &&Dummy&I;
    %END;
  %END;

* Merge variable &KeyValue with unique dummy names with original data;
  PROC SORT DATA=&Data OUT=&Recoded; BY &Variable; RUN;
  DATA &Recoded;
    MERGE &Recoded &DumNames; BY &Variable;
  RUN;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* C R E A T E   A N D   A S S I G N   D U M M Y   V A R I A B L E S ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;

* Sort input dataset into output dataset by &KeyValue to use FIRST. and LAST.;
  PROC SORT DATA=&Recoded; BY &Variable /*&KeyValue*/; RUN;

* Add dummy variables to sorted output dataset and assign Response values ;
  DATA &Recoded (DROP=&IndexVal Redefine _OldVar_ _NewVar_:
                      %IF (NOT &ValueKey) %THEN &KeyValue;) ;
    SET &Recoded END=LastCase; BY &Variable /*&KeyValue*/;
* Prepare labeling of dummy variables with values of &Variable;
    IF (_N_ EQ 1) THEN CALL EXECUTE ('DATA &Out;SET &Recoded; LABEL ');

* Create array of dummy variables to be addressed by index value;
    ARRAY Dummy %DO I = 1 %TO &DumTotal; &&Dummy&I %END; ; * array of dummies;
* Initialise all indicator dummy variables with non-response value for every record;
    DO _I_ = 1 TO &DumTotal;
      Dummy[_I_] = &NonResp;
    END;

* The sorted order of &KeyValue determines the index value to use to assign the dummies;
    RETAIN &IndexVal 0;
* Increase Index by 1 for every new &KeyValue value, also label concerning dummy variable;
    IF (FIRST.&Variable /*&KeyValue*/) THEN DO;
      &IndexVal+1; * Increase IndexVal by 1 at each new category value;
* Define original value of &Variable as label of Dummy[&IndexVal] variable;
      %IF (&VarType EQ C) %THEN %DO; %* Character &Variable;
        CALL EXECUTE (VNAME(Dummy[&IndexVal])||'="'||SUBSTRN(LEFT(&Variable),1,&MaxLabel)||'" ');
      %END;
      %ELSE %DO; %* Numeric &Variable;
        CALL EXECUTE (VNAME(Dummy[&IndexVal])||'="'||SUBSTRN(LEFT(PUT(&Variable,30.14)),1,&MaxLabel)||'" ');
      %END;
    END;

* Assign dummy variables, via the array elements, with the Response values;
    Dummy[&IndexVal] = &Response;
* If original &Variable value was missing (char ' ');
* then the dummy name is _ and its value may become &Missing, if specified;
    %IF (%LENGTH(%SUPERQ(Missing))) %THEN
      IF (MISSING(&Variable)) THEN Dummy[&IndexVal] = &Missing; ;

* Finalyse labeling of dummy variables with values of &Variable;
    IF (LastCase) THEN CALL EXECUTE ('; RUN; ');
  RUN;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Process error conditions ;
%*__________________________;

  %Aborting:

  %IF ( &ErrCount GT 0 ) %THEN %DO;
    %IF (%SYSFUNC(EXIST(&Out)) AND
         %FirstCh(&OverWrit) EQ Y) %THEN %DO;
      PROC DATASETS; DELETE &Out; RUN; QUIT;
    %END;
    %LET SYSLAST=&Data; * reset to original SYSLAST value;
    DATA _NULL_;
      ABORT /*ABEND*/;
    RUN;
  %END;
  %ELSE
    %LET SYSLAST=&Out; * reset to explicit SYSLAST value;

  %Finish:
 
%* --- Remove &Recoded &DumNames;
  PROC DATASETS NOLIST LIBRARY=WORK; DELETE &Recoded &DumNames; QUIT;
%*============;
%MEND DummyGen; %*===============================================================;
%*============;

%*------------------;
%* Auxiliary macros ; %* (attached) _Random_, _Length_ ; %* becoming external;
%*------------------;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%* Macro _Random_ generates a (unique) valid random SAS name of legal length ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - ;
%MACRO _Random_ (NameType=); %* NameType = DATASET / VARNAME;
  %LOCAL Value;
  %DO %UNTIL (NOT (%UPCASE(&NameType) EQ DATASET AND %SYSFUNC(EXIST(&Value))));
    %LET Value = %SYSFUNC(SLEEP(1,.01)); %* Pause for .01 second causing another name;
    %LET Value =
      _%SYSFUNC(SUBSTRN(%SYSFUNC(TRANSLATE(%SYSFUNC(TIME()),_,.:)),1,&MaxName-1));
  %END;
  &Value /* no semicolon, returned value */
%MEND _Random_; %* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = ;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Macro  _L e n g t h_  D E T E R M I N E s   M A X   N A M E   L E N G T H S ;
%*-----------------------------------------------------------------------------;
%* Adaptation of variable name and label length limits to SAS version (6,7,8,9);
%*_____________________________________________________________________________;
%MACRO _Length_;
  %GLOBAL  /* to pass on information upwards */
  MaxName  /* maximum variable name length */
  MaxLabel /* maximum variable label length */
  ValidVar /* ValidVarName option value */
  ;

  %IF (%SUBSTR(&SYSVER, 1, 2) EQ %STR(6.)) %THEN %DO; %* vs. 6.xx;
    %LET MaxName = 8;
    %LET MaxLabel = 40;
  %END;
  %ELSE %IF (%SUBSTR(&SYSVER, 1, 2) EQ %STR(8.)
          OR %SUBSTR(&SYSVER, 1, 2) EQ %STR(9.)) %THEN %DO; %* vs. 8.x.y, 9.x.y;
    %LET ValidVar = %UPCASE(%SYSFUNC(GETOPTION(VALIDVARNAME)));
    %IF (&ValidVar EQ V6) %THEN %DO;
      %LET MaxName = 8;
      %LET MaxLabel = 40;
    %END;
    %ELSE /* V7 or UPCASE */
    %DO;
      %LET MaxName = 32;
      %LET MaxLabel = 256;
    %END;
  %END;
  %ELSE
  %DO; %* any other vs.;
    %LET MaxName = 8;
    %LET MaxLabel = 40;
  %END;
%MEND _Length_; %* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = ;
